@@ -40,6 +40,9 @@ import (
 	ctrl "sigs.k8s.io/controller-runtime"
 	"sigs.k8s.io/controller-runtime/pkg/client/config"
 
+	apiconfigv1 "github.com/openshift/api/config/v1"
+	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
+
 	jenkinsv1alpha2 "github.com/jenkinsci/jenkins-automation-operator/api/v1alpha2"
 	"github.com/jenkinsci/jenkins-automation-operator/controllers"
 	"github.com/jenkinsci/jenkins-automation-operator/pkg/configuration/base"
@@ -71,6 +74,9 @@ func init() {
 	utilruntime.Must(routev1.AddToScheme(scheme))
 	utilruntime.Must(corev1.AddToScheme(scheme))
 	utilruntime.Must(monitoringv1.AddToScheme(scheme))
+	utilruntime.Must(apiextensionsv1.AddToScheme(scheme))
+	utilruntime.Must(apiconfigv1.AddToScheme(scheme))
+
 	// +kubebuilder:scaffold:scheme
 }
 
@@ -97,14 +103,16 @@ func main() {
 	}
 	checkRouteAPIAvailable(clientSet)
 	checkPrometheusAPIAvailable(clientSet)
+	checkProxyAPIAvailable(manager, clientSet)
 	notificationsChannel := make(chan e.Event)
 	go notifications.Listen(notificationsChannel, eventsRecorder, client)
 
 	// setup Jenkins controller
 	setupJenkinsRenconciler(manager, notificationsChannel)
 	setupJenkinsImageRenconciler(manager)
-	setupJenkinsBackupRenconciler(manager)
-	setupJenkinsRestoreRenconciler(manager)
+	setupJenkinsBackupRenconciler(manager, notificationsChannel)
+	setupJenkinsRestoreRenconciler(manager, notificationsChannel)
+	setupBackupVolumeRenconciler(manager, notificationsChannel)
 	// start the Cmd
 	setupLog.Info("Starting the Cmd.")
 	runMananger(manager)
@@ -130,6 +138,16 @@ func checkPrometheusAPIAvailable(clientSet *kubernetes.Clientset) {
 	}
 }
 
+func checkProxyAPIAvailable(manager manager.Manager, clientSet *kubernetes.Clientset) {
+	if resources.IsOpenshiftProxyAPIAvailable(clientSet) {
+		setupLog.Info("Proxy API found")
+		err := resources.QueryProxyConfig(manager)
+		if err != nil {
+			setupLog.Info("failed to query cluster proxy configuration")
+		}
+	}
+}
+
 func getEventsRecorder(cfg *rest.Config, debug *bool) event.Recorder {
 	events, err := event.New(cfg, constants.OperatorName)
 	if err != nil {
@@ -205,8 +223,8 @@ func getWatchNamespace() string {
 	return ns
 }
 
-func setupJenkinsRenconciler(mgr manager.Manager, channel chan e.Event) {
-	if err := newJenkinsReconciler(mgr, channel).SetupWithManager(mgr); err != nil {
+func setupJenkinsRenconciler(mgr manager.Manager, eventChan chan e.Event) {
+	if err := newJenkinsReconciler(mgr, eventChan).SetupWithManager(mgr); err != nil {
 		setupLog.Error(err, "unable to create controller", "controller", "Jenkins")
 		os.Exit(1)
 	}
@@ -236,33 +254,51 @@ func newJenkinsImageRenconciler(mgr manager.Manager) *controllers.JenkinsImageRe
 	}
 }
 
-func setupJenkinsBackupRenconciler(mgr manager.Manager) {
-	if err := newJenkinsBackupRenconciler(mgr).SetupWithManager(mgr); err != nil {
+func setupJenkinsBackupRenconciler(mgr manager.Manager, eventChan chan e.Event) {
+	if err := newJenkinsBackupRenconciler(mgr, eventChan).SetupWithManager(mgr); err != nil {
 		setupLog.Error(err, "unable to create controller", "controller", "Backup")
 		os.Exit(1)
 	}
 }
 
-func newJenkinsBackupRenconciler(mgr manager.Manager) *controllers.BackupReconciler {
+func newJenkinsBackupRenconciler(mgr manager.Manager, eventChan chan e.Event) *controllers.BackupReconciler {
 	return &controllers.BackupReconciler{
-		Client: mgr.GetClient(),
-		Log:    ctrl.Log.WithName("controllers").WithName("Backup"),
-		Scheme: mgr.GetScheme(),
+		Client:             mgr.GetClient(),
+		Log:                ctrl.Log.WithName("controllers").WithName("Backup"),
+		Scheme:             mgr.GetScheme(),
+		NotificationEvents: eventChan,
 	}
 }
 
-func setupJenkinsRestoreRenconciler(mgr manager.Manager) {
-	if err := newJenkinsRestoreRenconciler(mgr).SetupWithManager(mgr); err != nil {
+func setupJenkinsRestoreRenconciler(mgr manager.Manager, eventChan chan e.Event) {
+	if err := newJenkinsRestoreRenconciler(mgr, eventChan).SetupWithManager(mgr); err != nil {
 		setupLog.Error(err, "unable to create controller", "controller", "Restore")
 		os.Exit(1)
 	}
 }
 
-func newJenkinsRestoreRenconciler(mgr manager.Manager) *controllers.RestoreReconciler {
+func newJenkinsRestoreRenconciler(mgr manager.Manager, eventChan chan e.Event) *controllers.RestoreReconciler {
 	return &controllers.RestoreReconciler{
-		Client: mgr.GetClient(),
-		Log:    ctrl.Log.WithName("controllers").WithName("Restore"),
-		Scheme: mgr.GetScheme(),
+		Client:             mgr.GetClient(),
+		Log:                ctrl.Log.WithName("controllers").WithName("Restore"),
+		Scheme:             mgr.GetScheme(),
+		NotificationEvents: eventChan,
+	}
+}
+
+func setupBackupVolumeRenconciler(mgr manager.Manager, eventChan chan e.Event) {
+	if err := newBackupVolumeRenconciler(mgr, eventChan).SetupWithManager(mgr); err != nil {
+		setupLog.Error(err, "unable to create controller", "controller", "BackupVolume")
+		os.Exit(1)
+	}
+}
+
+func newBackupVolumeRenconciler(mgr manager.Manager, eventChan chan e.Event) *controllers.BackupVolumeReconciler {
+	return &controllers.BackupVolumeReconciler{
+		Client:             mgr.GetClient(),
+		Log:                ctrl.Log.WithName("controllers").WithName("BackupVolume"),
+		Scheme:             mgr.GetScheme(),
+		NotificationEvents: eventChan,
 	}
 }
 
@@ -276,7 +312,7 @@ func fatal(err error, debug bool) {
 }
 
 func printInfo() {
-	version.Version = "0.7.1"
+	version.Version = "0.7.2"
 	setupLog.Info(fmt.Sprintf("Version: %s", version.Version))
 	file, _ := filepath.Abs(os.Args[0])
 	setupLog.Info(fmt.Sprintf("MD5 checkcsum: %s", md5sum(file)))
